<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WDS_Helpers.
 *
 * @class    Iconic_WDS_Helpers
 * @version  1.0.0
 * @category Class
 * @author   Iconic
 */
class Iconic_WDS_Helpers {
	/**
	 * Run.
	 */
	public static function run() {
		self::add_filters();
	}

	/**
	 * Add filters.
	 */
	public static function add_filters() {
		add_filter( 'woocommerce_form_field_hidden', array( __CLASS__, 'form_field_hidden' ), 10, 4 );
	}

	/**
	 * Output hidden form field.
	 *
	 * @param string $field
	 * @param        $key
	 * @param        $args
	 * @param        $value
	 *
	 * @return string
	 */
	public static function form_field_hidden( $field, $key, $args, $value ) {
		$field .= '<input type="hidden" class="input-text ' . esc_attr( implode( ' ', $args['input_class'] ) ) . '" name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" placeholder="' . esc_attr( $args['placeholder'] ) . '"  value="' . esc_attr( $value ) . '" />';

		return $field;
	}

	/**
	 * Get time formatted.
	 *
	 * @param string $time Hi formatted time
	 *
	 * @return string
	 */
	public static function get_time_formatted( $time ) {
		global $jckwds;

		$time_format = $jckwds->settings['timesettings_timesettings_setup_timeformat'];
		$time        = DateTime::createFromFormat( 'Hi', str_pad( $time, 4, "0", STR_PAD_LEFT ) );

		return $time->format( $time_format );;
	}

	/**
	 * Get date formatted.
	 *
	 * @param $date Y-m-d H:i:s
	 *
	 * @return string
	 */
	public static function get_date_formatted( $date ) {
		$date_format = get_option( 'date_format' );
		$date        = new DateTime( $date );

		return $date->format( $date_format );
	}

	/**
	 * Get email link HTML.
	 *
	 * @param string $email
	 *
	 * @return string
	 */
	public static function get_email_link_html( $email ) {
		return sprintf( '<a href="mailto:%s" target="_blank">%s</a>', esc_url( $email ), $email );
	}

	/**
	 * Get user's name.
	 *
	 * @param WP_User $user
	 *
	 * @return string
	 */
	public static function get_user_name( $user ) {
		$full_name = implode( ' ', array(
			$user->first_name,
			$user->last_name,
		) );

		return ! empty( $full_name ) ? $full_name : $user->user_login;
	}

	/**
	 * Get label.
	 *
	 * @param string|bool $type
	 * @param null        $order
	 *
	 * @return bool|string|array
	 */
	public static function get_label( $type = false, $order = null ) {
		// Keep individual strings filtered for
		// backwards compatibility.
		$labels = apply_filters( 'iconic_wds_labels', array(
			'details'           => apply_filters( 'iconic_wds_delivery_details_text', __( 'Delivery Details', 'jckwds' ), $order ),
			'date'              => apply_filters( 'iconic_wds_delivery_date_text', __( 'Delivery Date', 'jckwds' ), $order ),
			'select_date'       => apply_filters( 'iconic_wds_select_delivery_date_text', __( 'Select a delivery date', 'jckwds' ), $order ),
			'choose_date'       => apply_filters( 'iconic_wds_choose_delivery_date_text', __( 'Please choose a date for your delivery.', 'jckwds' ), $order ),
			'select_date_first' => apply_filters( 'iconic_wds_select_date_first_text', __( 'Please select a date first...', 'jckwds' ) ),
			'time_slot'         => apply_filters( 'iconic_wds_time_slot_text', __( 'Time Slot', 'jckwds' ), $order ),
			'choose_time_slot'  => apply_filters( 'iconic_wds_choose_time_slot_text', __( 'Please choose a time slot for your delivery.', 'jckwds' ), $order ),
			'select_time_slot'  => apply_filters( 'iconic_wds_select_time_slot_text', __( 'Please select a time slot...', 'jckwds' ), $order ),
			'no_time_slots'     => apply_filters( 'iconic_wds_no_slots_available_text', __( 'Sorry, no slots available...', 'jckwds' ) ),
		), $order );

		if ( ! $type ) {
			return $labels;
		}

		if ( empty( $labels[ $type ] ) ) {
			return false;
		}

		return $labels[ $type ];
	}
}