<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WDS_Settings.
 *
 * @class    Iconic_WDS_Settings
 * @version  1.0.0
 * @author   Iconic
 */
class Iconic_WDS_Settings {
	/**
	 * Run.
	 */
	public static function run() {
		add_action( 'init', array( __CLASS__, 'init_settings' ) );
		add_action( 'admin_enqueue_scripts', array( __CLASS__, 'assets' ), 20 );
	}

	/**
	 * Init settings class.
	 */
	public static function init_settings() {
		global $jckwds;
		$jckwds->set_settings( Iconic_WDS_Core_Settings::$settings );
	}

	/**
	 * Scripts and styles on settings page.
	 */
	public static function assets() {
		$screen    = get_current_screen();
		$screen_id = $screen ? $screen->id : '';

		if ( $screen_id !== 'woocommerce_page_jckwds-settings' ) {
			return;
		}

		wp_enqueue_style( 'woocommerce_admin_styles' );
		wp_enqueue_script( 'wc-enhanced-select' );
	}

	/**
	 * Get category options.
	 *
	 * @return array|null
	 */
	public static function get_category_options() {
		static $categories = null;

		if ( ! is_null( $categories ) ) {
			return $categories;
		}

		$categories       = array();
		$categories_query = get_terms( array(
			'taxonomy' => 'product_cat',
		) );

		if ( empty( $categories_query ) ) {
			return $categories;
		}

		foreach ( $categories_query as $category ) {
			$categories[ $category->term_id ] = $category->name;
		}

		return $categories;
	}

	/**
	 * Get exclude products custom field.
	 *
	 * @return string
	 */
	public static function get_exclude_products_field() {
		ob_start();

		$exclude_products = wpsf_get_setting( 'jckwds', 'general_setup', 'exclude_products' );

		?>
		<select class="wc-product-search" multiple="multiple" style="width: 25em;" id="exclude_products" name="jckwds_settings[general_setup_exclude_products][]" data-sortable="true" data-placeholder="<?php esc_attr_e( 'Search for a product&hellip;', 'woocommerce' ); ?>" data-action="woocommerce_json_search_products">
			<?php
			$product_ids = ! empty( $exclude_products ) ? $exclude_products : array();

			foreach ( $product_ids as $product_id ) {
				$product = wc_get_product( $product_id );
				if ( is_object( $product ) ) {
					echo '<option value="' . esc_attr( $product_id ) . '"' . selected( true, true, false ) . '>' . wp_kses_post( $product->get_formatted_name() ) . '</option>';
				}
			}
			?>
		</select>
		<?php

		return ob_get_clean();
	}

	public static function get_day_fees_fields() {
		ob_start();

		$day_fees = self::get_day_fees();
		$days     = array(
			0 => __( 'Sunday', 'jckwds' ),
			1 => __( 'Monday', 'jckwds' ),
			2 => __( 'Tuesday', 'jckwds' ),
			3 => __( 'Wednesday', 'jckwds' ),
			4 => __( 'Thursday', 'jckwds' ),
			5 => __( 'Friday', 'jckwds' ),
			6 => __( 'Saturday', 'jckwds' ),
		);
		?>
		<ul class="wpsf-list wpsf-list--inputs">
			<?php foreach ( $day_fees as $day => $fee ) { ?>
				<li>
					<label style="display: inline-block;">
						<input type="number" name="jckwds_settings[datesettings_fees_days][<?php echo esc_attr( $day ); ?>]" value="<?php if ( ! empty( $fee ) ) { echo esc_attr( $fee ); } ?>" style="max-width: 120px; margin: 0 10px 0 0; vertical-align: middle;"> <?php echo $days[ $day ]; ?>
					</label>
				</li>
			<?php } ?>
		</ul>
		<?php

		return ob_get_clean();
	}

	/**
	 * Get day fees.
	 *
	 * @return array
	 */
	public static function get_day_fees() {
		$day_fees_setting = wpsf_get_setting( 'jckwds', 'datesettings_fees', 'days' );

		if ( empty( $day_fees_setting ) ) {
			return array(
				0 => 0,
				1 => 0,
				2 => 0,
				3 => 0,
				4 => 0,
				5 => 0,
				6 => 0,
			);
		}

		return array_map( 'floatval', $day_fees_setting );
	}
}